function B = subsref(A,S)

% B = SUBSREF(A,S)                 Jeffrey J. DaCunha 9/01/2003
%                                  Ian A. Gravagne 12/27/2003
%
% Returns the operation A(1) or A(2:4) where A is a timescale object. 
% Output is a vector the same length as the input. Also returns the
% timescale data and type arrays as A.data and A.type
% 
% A(m:n)      Returns a row vector containing the data in A from index 
%             m to n regardless of interval boundaries.
% A(m:n,k)    Returns a row vector containing the data in the kth interval
%             of A. 
% A(m:n,p:q)  Returns an error. 
%
% A{n}        Returns a timescale object containing the data of interval n.
%             This is a shortcut for
%               S = size(A);
%               T = timescale(A(:,n), 'd' or 'c')
%

%%%% PRELIMINARYT ERROR CHECK %%%%
if(size(S,2) > 1)
    error('Timescale object cannot be multiply dereferenced.');
end

%%%% IF DEREFERENCE IS () %%%%
if(isequal(S(1).type,'()'))

    % Error check for () case
    if(length(S.subs) > 2)
        error('Parenthesis operator can only dereference 2 dimensions (data and interval).');
    end
    
    % Paren dereference handler
    if(length(S.subs) == 1)             
        
        if(S.subs{1} == ':')                % User entered A(:)
            B = A.data;
        else                                % User entered A(2) or A(3:5)
            if( max(S.subs{1}) > length(A.data) )
                error('Index of indices out of bounds for timescale.');
            end
            B = A.data(S.subs{1});
        end
        
    else                                    % User entered A(:,2) or A(4:6,3)
    
        type = [0,A.type(1,:)];
        
        if( (S.subs{2} == ':') | (length(S.subs{2}) > 1) )
            error('Timescale object can only have integer dereference for 2nd dimension.');
        elseif(S.subs{2} > size(A.type,2))
            message = ['Timescale object does not have ',num2str(S.subs{2}),' intervals.'];
            error(message);
        end
        
        if(S.subs{1} == ':')
            p = type(S.subs{2}) + 1;
            q = type(S.subs{2}+1);
            B = A.data(p:q);
        else
            p = type(S.subs{2}) + 1;
            q = type(S.subs{2}+1);
            if( (length(S.subs{1}) > (q-p+1)) | max(S.subs{1}) > q )
                message = ['Index or indices out of bounds for timescale interval ',num2str(S.subs{2}),'.'];
                error(message);
            end
            B = A.data(S.subs{1} + p - 1);
        end
        
    end

%%%% IF DEREFERENCE IS {} %%%%
elseif(isequal(S(1).type,'{}'))
    
    % Error check for {}

    if(length(S.subs) > 1)
        error('Bracket operator {} can only dereference 1 dimension (the timescale interval).');
    end
    
    % Dereference handler for bracket
    if(S.subs{1} == ':')
    
        B = A;
        
    else
  
        if( S.subs{1} > size(A.type,2) )
            error('Index is greater than number of intervals');
        elseif( length(S.subs{1}) > 1 )
            error('Index must be an integer or a colon');
        end
        
        type = [0,A.type(1,:)];
        i = S.subs{1};
        p = type(i) + 1;
        q = type(i + 1);
        data = A.data(p:q);
        s = [length(data);A.type(2,i)];
        B = timescale(data,s);
        
    end

%%%% IF DEREFERENCE IS "dot" %%%%
elseif(isequal(S(1).type,'.'))
    
    error('Timescale object fields cannot be accessed directly.');

%%%% ANY OTHER CASES %%%%
else
    
    error('Timescale reference could not be interpreted. See HELP timescale/subsref');
    
end
