function Z = tsode(f,T,y0)

% TSODE - Timescale First Order Ordinary Differential Equation Solver
%   Z = TSODE(ODEFUN,T,Y0) integrates the system of differential
%   equations y' = f(t,y) from time T0 to TFINAL in the timescale T with
%   initial conditions Y0. Function ODEFUN(T,Y) must return a column vector
%   corresponding to f(t,y). The output Z is a cell array containing the
%   resulting timescales. Each value in the solution timescales Y
%   corresponds to a time in the timescale T. When the interval is
%   continuous, the solution is obtained using ODE23 which solves non-stiff
%   differential equations using the low order method. When the interval is
%   discrete, the solution is obtained by a recursive difference method:
%   
%                   y(t+1) = f(y(t))*h + y(t)
%                    
%   where y(t+1) is the next value in the solution timescale, f(y(t)) is
%   the result of the current solution passed through the function f, and h
%   is the graininess of the timescale T.
%                                                                 
%   The output Z is a cell array containing the resulting timescales Y. For
%   example, for a second order differential equation, the output displayed
%   output Z will be:
%   
%                   Z = [6-D timescale]    [6-D timescale]
%               
%   To obtain each resulting timescale Y1 and Y2, Z must be accessed thus:
%   
%                   Y1 = Z{1}, Y2 = Z{2}
%                                                                        
%   USEFUL EXAMPLE
%   For a second order differential equation, f(t,y) is defined in the
%   file, 'f.m' below (ODEFUN):
%                   function dy = f(t,y)
%
%                   y1 = y(1); y2 = y(2);
%
%                   dy1 = y1+y2;
%                   dy2 = -y2 + cos(y1);
%
%                   dy = [dy1;dy2];
%
%   Define input timescale:
%
%                   T = timescale([2:1:10],'c',[11 12 13],'d');
%
%   Define y0 a column vector (because second order):
%                
%                   y0 = [1;1];
%
%   At command prompt, to yield the solution cell array:
%
%                   Z = tsode(@f,T,y0);
%
%   The result is a cell array Z containing two timescales Y1 and Y2,
%   accessed by typing:
%
%                   Y1 = Z{1}, Y2 = Z{2}
%
%
%
%   Brian Ballard, Bunmi Otegbade   07/15/05

clear Z
type = size(T);

% ERROR CHECKING

% Checking number of arguments
    if(nargin < 3 | nargin > 3)
        error('Requires 3 arguments.')
    end
    
% Checking if T is a valid timescale 
    if(~isa(T,'timescale'))
        error('Second argument must be a timescale object')
    end

% Checking if T is empty
    if (type(1)==0)
        error('All timescale intervals must be non-empty')
    end
    
    
%Define the number of output TS. typey
typey = size(y0);
N = typey(1);

%Modeling the TS form type

Y = timescale(zeros(1,length(T)),type);
Z = cell(1,N);

%Format cell with same number of TS
for r = 1:N
    Z{r} = Y;
end


p = 1;
for i = 1:size(type,2)
    q = type(1,i);
    
    %Check if EMPTY interval
    if( isempty(T(:,i)) )
        error('Intervals cannot be empty');
    end
    %end EMPTY interval
    
    % DISCRETE
    if (type(2,i))
        
        v = T(:,i);     % Strip members of interval into a vector 
        mu = diff(v);
                
        % Is this the first interval(DISCRETE)
        if(i < 2)
           
            % Implemented this way, we're basically re-creating the
            % operation of ode23. Sending out a matrix that also reports
            % the initial value.
            hold = y0;
            %Put first values in cell array
            %for j = 1:N
            %     Z{j}(1,1) = y0(j);
            %end
            %Z{1}(1,1),Z{2}(1,1)
            % When first order, solution is a row vector; when Nth, matrix
            solution = oderecursive(p+1,q,mu,hold,f,T,i,N);
            for j = 1:N
                Z{j}((1:q-p+1),i) = solution(j,:); %Throw in each row.
            end
            
            
            
            %RETAINING LAST VALUES
            for j = 1:N
                hold(j) = Z{j}((q-p+1),i);    % Keep last results for glue
            end
            last = T(q);            % Keep last t value for mul
            
        else % Not the first interval
            first  = T(p);
            % Calculate Glue
            mul = first - last;          
            hold = mul * f(last,hold,mu) + hold;           
            
            %Put first value of i th interval in cell array
            %for j = 1:N
            %    Z{j}(1,i) = hold(j);
            %end
            
            % Does all N output timescales
            solution = oderecursive(p+1,q,mu,hold,f,T,i,N);    
            for j = 1:N
                Z{j}((1:q-p+1),i) = solution(j,:);
            end
            
            %RETAINING LAST VALUES
            for j = 1:N
                hold(j) = Z{j}((q-p+1),i);    % Keep last result for glue
            end
            last = T(q);            % Keep last t value for mul
        end 
    
    else % Continuous
        
        if(i < 2)
            
            [discard,w] = ode23(f,T(p:q),y0);
            w = w.'; % transpose
            % All points in the interval
            for j = 1:N
                Z{j}(:,i) = w(j,:);
            end
            
            %RETAINING LAST VALUES
            for j = 1:N
                hold(j) = Z{j}((q-p+1),i);    % Keep last result for glue
            end
            last = T(q);            % Keep last t value for mul
            
        else
            first  = T(p);
            mul = first - last;
            %hold = hold.'; % Transpose hold because row vec(need col vec)
            %size(hold);
            hold = mul * f(last,hold) + hold; %Don't put this in output;send to ODE23
            
            % ERROR CHECK: ODE23 INVENTS STEPS ON 3 CONTENT INT
            typez = size(T(:,i));
            x = typez(2);             
            if(x <= 3)
                error('Step size of interval must be specified or interval must contain more than three values')
            end
           
            %Put remaining values of i-th interval in cell array 
            [discard,m] = ode23(f,T(p:q),hold);
            m = m.';
            for j = 1:N            
                Z{j}((1:q-p+1),i) = m(j,:);
            end
           
            %RETAINING LAST VALUES
            for j = 1:N
                hold(j) = Z{j}((q-p+1),i);    % Keep last result for glue
            end
            last = T(q);            % Keep last t value for mul
            
        end
        
    end % END D OR C
    
    % Want column vectors instead of row vectors.
    szhd = size(hold);
    
    if(szhd(2)>szhd(1))
        hold = hold.';
    end
    
    p = q + 1;
end

